@@ -48,7 +48,7 @@ function mergeCFiles(inputs, outputFilePath)
 }
 
 function sanitizedList(list, product, fullPropertyName) {
-    if (!Array.isArray(list))
+    if (!(list instanceof Array))
         return list;
     var filterFunc = function(elem) {
         if (typeof elem === "string" && elem.length === 0) {
@@ -100,7 +100,7 @@ function artifactInstalledFilePath(artifact) {
   * found, an exception is thrown.
   */
 function fileTagForTargetLanguage(fileTags) {
-    var srcTags = ["c", "cpp", "objc", "objcpp", "asm", "asm_cpp"];
+    var srcTags = ["c", "cpp", "objc", "objcpp", "asm", "asm_cpp", "cppm"];
     var pchTags = ["c_pch", "cpp_pch", "objc_pch", "objcpp_pch"];
 
     var canonicalTag = undefined;
@@ -408,7 +408,7 @@ var PropertyValidator = (function () {
             errorMessage += "The following properties have invalid values:\n";
             lines = [];
             for (i in invalidProperties) {
-                for (j in invalidProperties[i]) {
+                for (j = 0; j < invalidProperties[i].length; ++j) {
                     lines.push(this.moduleName + "." + i + ": " + invalidProperties[i][j]);
                 }
             }
@@ -538,8 +538,13 @@ function guessArchitecture(m) {
                         break;
                 }
             }
-        } else if (hasAnyOf(m, ["__i386", "__i386__", "_M_IX86"])) {
+        } else if (hasAnyOf(m, ["__i386", "__i386__", "__386__"])) {
             architecture = "x86";
+        } else if (hasAnyOf(m, ["_M_IX86"])) {
+            var code = parseInt(m["_M_IX86"]);
+            architecture = (code < 300) ? "x86_16" : "x86";
+        } else if (hasAnyOf(m, ["_M_I86"])) {
+            architecture = "x86_16";
         } else if (hasAnyOf(m, ["__x86_64", "__x86_64__", "__amd64", "_M_X64", "_M_AMD64"])) {
             architecture = "x86_64";
             if (hasAnyOf(m, ["__x86_64h", "__x86_64h__"]))
@@ -588,6 +593,12 @@ function guessArchitecture(m) {
             architecture = "sh";
         } else if (hasAnyOf(m, ["__CR16__"])) {
             architecture = "cr16";
+        } else if (hasAnyOf(m, ["__mc68hc1x__", "__mc68hc1x"])) {
+            architecture = "hcs12";
+        } else if (hasAnyOf(m, ["__e2k__"])) {
+            architecture = "e2k";
+        } else if (hasAnyOf(m, ["__hppa__"])) {
+            architecture = "hppa";
         }
     }
 
@@ -606,15 +617,19 @@ function guessTargetPlatform(m) {
             return "vxworks";
         if (hasAnyOf(m, ["__APPLE__"]))
             return "darwin";
-        if (hasAnyOf(m, ["WIN32", "_WIN32", "__WIN32__", "__NT__"]))
+        if (hasAnyOf(m, ["WIN32", "_WIN32", "__WIN32__", "__NT__", "__WINDOWS__", "_WINDOWS"]))
             return "windows";
+        if (hasAnyOf(m, ["MSDOS", "__DOS__", "DOS386"]))
+            return "dos";
+        if (hasAnyOf(m, ["__OS2__"]))
+            return "os2";
         if (hasAnyOf(m, ["_AIX"]))
             return "aix";
         if (hasAnyOf(m, ["hpux", "__hpux"]))
             return "hpux";
         if (hasAnyOf(m, ["__sun", "sun"]))
             return "solaris";
-        if (hasAnyOf(m, ["__linux__", "__linux"]))
+        if (hasAnyOf(m, ["__linux__", "__linux", "__LINUX__"]))
             return "linux";
         if (hasAnyOf(m, ["__FreeBSD__", "__DragonFly__", "__FreeBSD_kernel__"]))
             return "freebsd";
@@ -626,6 +641,8 @@ function guessTargetPlatform(m) {
             return "hurd";
         if (hasAnyOf(m, ["__HAIKU__"]))
             return "haiku";
+        if (hasAnyOf(m, ["__wasm__"]))
+            return "wasm-emscripten"
     }
 }
 
